@@ -19,7 +19,7 @@ from . import extensions
 from . import fingerprinting
 from . import headings
 from . import highlight
-from . import HTMLSerializer
+from . import html
 from . import idl
 from . import includes
 from . import inlineTags
@@ -29,7 +29,7 @@ from . import metadata
 from . import shorthands
 from . import wpt
 
-from .htmlhelpers import *
+from .html import *
 from .InputSource import InputSource, FileInputSource, StdinInputSource
 from .Line import Line
 from .messages import *
@@ -50,6 +50,7 @@ class Spec(object):
             die("No input file specified, and no *.bs or *.src.html files found in current directory.\nPlease specify an input file, or use - to pipe from STDIN.")
             return
         self.inputSource = InputSource(inputFilename)
+        self.transitiveDependencies = set()
         self.debug = debug
         self.token = token
         self.testing = testing
@@ -100,10 +101,13 @@ class Spec(object):
 
         return True
 
+    def recordDependencies(self, *inputSources):
+        self.transitiveDependencies.update(inputSources)
+
     def preprocess(self):
+        self.transitiveDependencies.clear()
         self.assembleDocument()
         self.processDocument()
-        return self
 
     def assembleDocument(self):
         # Textual hacks
@@ -111,6 +115,7 @@ class Spec(object):
         if self.lineNumbers:
             self.lines = hackyLineNumbers(self.lines)
         self.lines = markdown.stripComments(self.lines)
+        self.recordDependencies(self.inputSource)
         # Extract and process metadata
         self.lines, self.mdDocument = metadata.parse(lines=self.lines)
         # First load the metadata sources from 'local' data
@@ -154,7 +159,6 @@ class Spec(object):
         correctH1(self)
         includes.processInclusions(self)
         metadata.parseDoc(self)
-        return self
 
     def processDocument(self):
         # Fill in and clean up a bunch of data
@@ -253,7 +257,10 @@ class Spec(object):
         return self
 
     def serialize(self):
-        rendered = HTMLSerializer.HTMLSerializer(self.document, self.md.opaqueElements, self.md.blockElements).serialize()
+        try:
+            rendered = html.Serializer(self.md.opaqueElements, self.md.blockElements).serialize(self.document)
+        except e:
+            die("{0}", e)
         rendered = finalHackyCleanup(rendered)
         return rendered
 
@@ -277,7 +284,7 @@ class Spec(object):
         if not constants.dryRun:
             try:
                 if outputFilename == "-":
-                    sys.stdout.write(rendered.encode("utf-8"))
+                    sys.stdout.write(rendered)
                 else:
                     with io.open(outputFilename, "w", encoding="utf-8") as f:
                         f.write(rendered)
@@ -335,22 +342,25 @@ class Spec(object):
         mdCommandLine = self.mdCommandLine
 
         try:
-            lastInputModified = self.inputSource.mtime()
             self.preprocess()
             self.finish(outputFilename)
+            lastInputModified = {dep: dep.mtime()
+                                 for dep in self.transitiveDependencies}
             p("==============DONE==============")
             try:
                 while True:
-                    inputModified = self.inputSource.mtime()
-                    if inputModified > lastInputModified:
+                    # Comparing mtimes with "!=" handles when a file starts or
+                    # stops existing, and it's fine to rebuild if an mtime
+                    # somehow gets older.
+                    if any(input.mtime() != lastModified for input, lastModified in lastInputModified.items()):
                         resetSeenMessages()
-                        lastInputModified = inputModified
-                        formattedTime = datetime.fromtimestamp(inputModified).strftime("%H:%M:%S")
-                        p("Source file modified at {0}. Rebuilding...".format(formattedTime))
+                        p("Source file modified. Rebuilding...")
                         self.initializeState()
                         self.mdCommandLine = mdCommandLine
                         self.preprocess()
                         self.finish(outputFilename)
+                        lastInputModified = {dep: dep.mtime()
+                                             for dep in self.transitiveDependencies}
                         p("==============DONE==============")
                     time.sleep(1)
             except KeyboardInterrupt:
